# -*- coding: utf-8 -*-

# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/aiplatform/matching_engine/_protos/match_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.rpc import status_pb2 as google_dot_rpc_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\nCgoogle/cloud/aiplatform/matching_engine/_protos/match_service.proto\x12)google.cloud.aiplatform.container.v1beta1\x1a\x17google/rpc/status.proto"\x97\x02\n\x0cMatchRequest\x12\x19\n\x11\x64\x65ployed_index_id\x18\x01 \x01(\t\x12\x11\n\tfloat_val\x18\x02 \x03(\x02\x12\x15\n\rnum_neighbors\x18\x03 \x01(\x05\x12G\n\trestricts\x18\x04 \x03(\x0b\x32\x34.google.cloud.aiplatform.container.v1beta1.Namespace\x12,\n$per_crowding_attribute_num_neighbors\x18\x05 \x01(\x05\x12\x1c\n\x14\x61pprox_num_neighbors\x18\x06 \x01(\x05\x12-\n%leaf_nodes_to_search_percent_override\x18\x07 \x01(\x05"\x8e\x01\n\rMatchResponse\x12S\n\x08neighbor\x18\x01 \x03(\x0b\x32\x41.google.cloud.aiplatform.container.v1beta1.MatchResponse.Neighbor\x1a(\n\x08Neighbor\x12\n\n\x02id\x18\x01 \x01(\t\x12\x10\n\x08\x64istance\x18\x02 \x01(\x01"\x9f\x02\n\x11\x42\x61tchMatchRequest\x12h\n\x08requests\x18\x01 \x03(\x0b\x32V.google.cloud.aiplatform.container.v1beta1.BatchMatchRequest.BatchMatchRequestPerIndex\x1a\x9f\x01\n\x19\x42\x61tchMatchRequestPerIndex\x12\x19\n\x11\x64\x65ployed_index_id\x18\x01 \x01(\t\x12I\n\x08requests\x18\x02 \x03(\x0b\x32\x37.google.cloud.aiplatform.container.v1beta1.MatchRequest\x12\x1c\n\x14low_level_batch_size\x18\x03 \x01(\x05"\xac\x02\n\x12\x42\x61tchMatchResponse\x12k\n\tresponses\x18\x01 \x03(\x0b\x32X.google.cloud.aiplatform.container.v1beta1.BatchMatchResponse.BatchMatchResponsePerIndex\x1a\xa8\x01\n\x1a\x42\x61tchMatchResponsePerIndex\x12\x19\n\x11\x64\x65ployed_index_id\x18\x01 \x01(\t\x12K\n\tresponses\x18\x02 \x03(\x0b\x32\x38.google.cloud.aiplatform.container.v1beta1.MatchResponse\x12"\n\x06status\x18\x03 \x01(\x0b\x32\x12.google.rpc.Status"D\n\tNamespace\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0c\x61llow_tokens\x18\x02 \x03(\t\x12\x13\n\x0b\x64\x65ny_tokens\x18\x03 \x03(\t2\x9a\x02\n\x0cMatchService\x12|\n\x05Match\x12\x37.google.cloud.aiplatform.container.v1beta1.MatchRequest\x1a\x38.google.cloud.aiplatform.container.v1beta1.MatchResponse"\x00\x12\x8b\x01\n\nBatchMatch\x12<.google.cloud.aiplatform.container.v1beta1.BatchMatchRequest\x1a=.google.cloud.aiplatform.container.v1beta1.BatchMatchResponse"\x00\x62\x06proto3'
)


_MATCHREQUEST = DESCRIPTOR.message_types_by_name["MatchRequest"]
_MATCHRESPONSE = DESCRIPTOR.message_types_by_name["MatchResponse"]
_MATCHRESPONSE_NEIGHBOR = _MATCHRESPONSE.nested_types_by_name["Neighbor"]
_BATCHMATCHREQUEST = DESCRIPTOR.message_types_by_name["BatchMatchRequest"]
_BATCHMATCHREQUEST_BATCHMATCHREQUESTPERINDEX = _BATCHMATCHREQUEST.nested_types_by_name[
    "BatchMatchRequestPerIndex"
]
_BATCHMATCHRESPONSE = DESCRIPTOR.message_types_by_name["BatchMatchResponse"]
_BATCHMATCHRESPONSE_BATCHMATCHRESPONSEPERINDEX = (
    _BATCHMATCHRESPONSE.nested_types_by_name["BatchMatchResponsePerIndex"]
)
_NAMESPACE = DESCRIPTOR.message_types_by_name["Namespace"]
MatchRequest = _reflection.GeneratedProtocolMessageType(
    "MatchRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _MATCHREQUEST,
        "__module__": "google.cloud.aiplatform.matching_engine._protos.match_service_pb2"
        # @@protoc_insertion_point(class_scope:google.cloud.aiplatform.container.v1beta1.MatchRequest)
    },
)
_sym_db.RegisterMessage(MatchRequest)

MatchResponse = _reflection.GeneratedProtocolMessageType(
    "MatchResponse",
    (_message.Message,),
    {
        "Neighbor": _reflection.GeneratedProtocolMessageType(
            "Neighbor",
            (_message.Message,),
            {
                "DESCRIPTOR": _MATCHRESPONSE_NEIGHBOR,
                "__module__": "google.cloud.aiplatform.matching_engine._protos.match_service_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.aiplatform.container.v1beta1.MatchResponse.Neighbor)
            },
        ),
        "DESCRIPTOR": _MATCHRESPONSE,
        "__module__": "google.cloud.aiplatform.matching_engine._protos.match_service_pb2"
        # @@protoc_insertion_point(class_scope:google.cloud.aiplatform.container.v1beta1.MatchResponse)
    },
)
_sym_db.RegisterMessage(MatchResponse)
_sym_db.RegisterMessage(MatchResponse.Neighbor)

BatchMatchRequest = _reflection.GeneratedProtocolMessageType(
    "BatchMatchRequest",
    (_message.Message,),
    {
        "BatchMatchRequestPerIndex": _reflection.GeneratedProtocolMessageType(
            "BatchMatchRequestPerIndex",
            (_message.Message,),
            {
                "DESCRIPTOR": _BATCHMATCHREQUEST_BATCHMATCHREQUESTPERINDEX,
                "__module__": "google.cloud.aiplatform.matching_engine._protos.match_service_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.aiplatform.container.v1beta1.BatchMatchRequest.BatchMatchRequestPerIndex)
            },
        ),
        "DESCRIPTOR": _BATCHMATCHREQUEST,
        "__module__": "google.cloud.aiplatform.matching_engine._protos.match_service_pb2"
        # @@protoc_insertion_point(class_scope:google.cloud.aiplatform.container.v1beta1.BatchMatchRequest)
    },
)
_sym_db.RegisterMessage(BatchMatchRequest)
_sym_db.RegisterMessage(BatchMatchRequest.BatchMatchRequestPerIndex)

BatchMatchResponse = _reflection.GeneratedProtocolMessageType(
    "BatchMatchResponse",
    (_message.Message,),
    {
        "BatchMatchResponsePerIndex": _reflection.GeneratedProtocolMessageType(
            "BatchMatchResponsePerIndex",
            (_message.Message,),
            {
                "DESCRIPTOR": _BATCHMATCHRESPONSE_BATCHMATCHRESPONSEPERINDEX,
                "__module__": "google.cloud.aiplatform.matching_engine._protos.match_service_pb2"
                # @@protoc_insertion_point(class_scope:google.cloud.aiplatform.container.v1beta1.BatchMatchResponse.BatchMatchResponsePerIndex)
            },
        ),
        "DESCRIPTOR": _BATCHMATCHRESPONSE,
        "__module__": "google.cloud.aiplatform.matching_engine._protos.match_service_pb2"
        # @@protoc_insertion_point(class_scope:google.cloud.aiplatform.container.v1beta1.BatchMatchResponse)
    },
)
_sym_db.RegisterMessage(BatchMatchResponse)
_sym_db.RegisterMessage(BatchMatchResponse.BatchMatchResponsePerIndex)

Namespace = _reflection.GeneratedProtocolMessageType(
    "Namespace",
    (_message.Message,),
    {
        "DESCRIPTOR": _NAMESPACE,
        "__module__": "google.cloud.aiplatform.matching_engine._protos.match_service_pb2"
        # @@protoc_insertion_point(class_scope:google.cloud.aiplatform.container.v1beta1.Namespace)
    },
)
_sym_db.RegisterMessage(Namespace)

_MATCHSERVICE = DESCRIPTOR.services_by_name["MatchService"]
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _MATCHREQUEST._serialized_start = 140
    _MATCHREQUEST._serialized_end = 419
    _MATCHRESPONSE._serialized_start = 422
    _MATCHRESPONSE._serialized_end = 564
    _MATCHRESPONSE_NEIGHBOR._serialized_start = 524
    _MATCHRESPONSE_NEIGHBOR._serialized_end = 564
    _BATCHMATCHREQUEST._serialized_start = 567
    _BATCHMATCHREQUEST._serialized_end = 854
    _BATCHMATCHREQUEST_BATCHMATCHREQUESTPERINDEX._serialized_start = 695
    _BATCHMATCHREQUEST_BATCHMATCHREQUESTPERINDEX._serialized_end = 854
    _BATCHMATCHRESPONSE._serialized_start = 857
    _BATCHMATCHRESPONSE._serialized_end = 1157
    _BATCHMATCHRESPONSE_BATCHMATCHRESPONSEPERINDEX._serialized_start = 989
    _BATCHMATCHRESPONSE_BATCHMATCHRESPONSEPERINDEX._serialized_end = 1157
    _NAMESPACE._serialized_start = 1159
    _NAMESPACE._serialized_end = 1227
    _MATCHSERVICE._serialized_start = 1230
    _MATCHSERVICE._serialized_end = 1512
# @@protoc_insertion_point(module_scope)
